@@ -1,5 +1,5 @@
 /*
- *	$Id: ipfwadm.c,v 1.3 1995/04/08 13:27:33 jos Exp $
+ *	$Id: ipfwadm.c,v 1.5 1995/04/19 20:28:49 jos Exp $
  *
  *
  *	ipfwadm -- IP firewall and accounting administration
@@ -43,12 +43,16 @@
  *	1.2	Various changes in error messages, print format, comment.
  *	1.3	Add colon to 'p' in getopt string.
  *		Correct bug in printing range of destination port.
+ *	1.4	Change the usage messages and add a large help text for -h.
+ *	1.5	Change code to make "gcc -Wall" happy (and a few of these
+ *		warnings were really serious ...).
  */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <getopt.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <linux/ip.h>
@@ -98,7 +102,7 @@
 #define FMT_VIAHOST	0x0080
 #define FMT_NONEWLINE	0x0100
 
-char ipfwadm_version[] = "$Id: ipfwadm.c,v 1.3 1995/04/08 13:27:33 jos Exp $";
+char ipfwadm_version[] = "$Id: ipfwadm.c,v 1.5 1995/04/19 20:28:49 jos Exp $";
 
 int ssocmd_add[3] = {IP_FW_ADD_BLK, IP_FW_ADD_FWD, IP_ACCT_ADD};
 int ssocmd_delete[3] = {IP_FW_DEL_BLK, IP_FW_DEL_FWD, IP_ACCT_DEL};
@@ -162,12 +166,13 @@ extern void *fw_malloc(size_t);
 extern void *fw_calloc(size_t, size_t);
 extern void *fw_realloc(void *, size_t);
 extern void exit_error(int, char *);
-extern void exit_usage();
+extern void exit_tryhelp(int);
+extern void exit_printhelp();
 
 int
 main(int argc, char *argv[])
 {
-	int c, kind, ret, i, j, *ssocmd = NULL, policy, dummy;
+	int c, kind, ret = 0, policy, dummy;
 
 	program = argv[0];
 
@@ -175,22 +180,22 @@ main(int argc, char *argv[])
 		switch (c) {
 		case 'A':
 			if (chain != CHN_NONE)
-				exit_usage();
+				exit_error(2, "multiple categories specified");
 			chain = CHN_ACCT;
 			break;
 		case 'B':
 			if (chain != CHN_NONE)
-				exit_usage();
+				exit_error(2, "multiple categories specified");
 			chain = CHN_BLOCK;
 			break;
 		case 'F':
 			if (chain != CHN_NONE)
-				exit_usage();
+				exit_error(2, "multiple categories specified");
 			chain = CHN_FORWARD;
 			break;
 		case 'a':
 			if (command != CMD_NONE)
-				exit_usage();
+				exit_error(2, "multiple commands specified");
 			command = CMD_ADD;
 			if (optind < argc && argv[optind][0] != '-') {
 				parse_policy(argv[optind++]);
@@ -199,7 +204,7 @@ main(int argc, char *argv[])
 			break;
 		case 'd':
 			if (command != CMD_NONE)
-				exit_usage();
+				exit_error(2, "multiple commands specified");
 			command = CMD_DELETE;
 			if (optind < argc && argv[optind][0] != '-') {
 				parse_policy(argv[optind++]);
@@ -208,32 +213,32 @@ main(int argc, char *argv[])
 			break;
 		case 'l':
 			if (command != CMD_NONE && command != CMD_RESET)
-				exit_usage();
+				exit_error(2, "multiple commands specified");
 			command |= CMD_LIST;
 			break;
 		case 'z':
 			if (command != CMD_NONE && command != CMD_LIST)
-				exit_usage();
+				exit_error(2, "multiple commands specified");
 			command |= CMD_RESET;
 			break;
 		case 'f':
 			if (command != CMD_NONE)
-				exit_usage();
+				exit_error(2, "multiple commands specified");
 			command = CMD_FLUSH;
 			break;
 		case 'p':
 			if (command != CMD_NONE)
-				exit_usage();
+				exit_error(2, "multiple commands specified");
 			command = CMD_POLICY;
 			parse_policy(optarg);
 			break;
 		case 'c':
 			if (command != CMD_NONE)
-				exit_usage();
+				exit_error(2, "multiple commands specified");
 			command = CMD_CHECK;
 			break;
 		case 'h':
-			exit_usage();
+			exit_printhelp();
 			/* we'll never reach this */
 			break;
 		case 'P':
@@ -295,7 +300,7 @@ main(int argc, char *argv[])
 			break;
 		case '?':
 		default:
-			exit_usage();
+			exit_tryhelp(2);
 		}
 
 	if (optind < argc)
@@ -403,7 +408,8 @@ main(int argc, char *argv[])
 		ret = list_entries(chain, (command & CMD_RESET) ? "r+" : "r");
 		break;
 	default:
-		exit_usage();
+		/* We should never reach this... */
+		exit_tryhelp(2);
 	}
 	exit(ret);
 }
@@ -613,8 +619,6 @@ parse_port(char *port, unsigned short kind)
 void
 store_port(char *port, unsigned short *nports, int rangeflag, char *ports[])
 {
-	int newports;
-
 	/* to count the # ports, check whether this is a range or not */
 	if (strchr(port, (int) ':') == NULL)
 		*nports += 1;
@@ -855,7 +859,7 @@ add_delete_entries(int cmd, int chain)
 			ret |= do_setsockopt(cmd, &firewall, sizeof(firewall));
 		}
 	}
-
+	return ret;
 }
 
 int
@@ -1123,7 +1127,7 @@ read_procinfo(FILE *fp, struct ip_fw *fwlist, int nfwlist)
 
 	for (nread = 0; nread < nfwlist; nread++) {
 		fw = &fwlist[nread];
-		if ((n = fscanf(fp, "%lX/%lX->%lX/%lX %lX %X %u %u %lu %lu",
+		if ((n = fscanf(fp, "%lX/%lX->%lX/%lX %lX %hX %hu %hu %lu %lu",
 				&fw->fw_src.s_addr, &fw->fw_smsk.s_addr, 
 				&fw->fw_dst.s_addr, &fw->fw_dmsk.s_addr, 
 				&fw->fw_via.s_addr, &fw->fw_flg,
@@ -1134,7 +1138,7 @@ read_procinfo(FILE *fp, struct ip_fw *fwlist, int nfwlist)
 			exit_error(1, "unexpected input data");
 		else
 			for (i = 0; i < IP_FW_MAX_PORTS; i++)
-				if (fscanf(fp, "%u", &fw->fw_pts[i]) != 1)
+				if (fscanf(fp, "%hu", &fw->fw_pts[i]) != 1)
 					exit_error(1, "unexpected input data");
 
 		/* we always keep these addresses in network byte order */
@@ -1144,6 +1148,7 @@ read_procinfo(FILE *fp, struct ip_fw *fwlist, int nfwlist)
 		fw->fw_smsk.s_addr = htonl(fw->fw_smsk.s_addr);
 		fw->fw_dmsk.s_addr = htonl(fw->fw_dmsk.s_addr);
 	}
+	return nread;
 }
 
 struct ip_fwpkt *
@@ -1280,16 +1285,47 @@ void
 exit_error(int status, char *msg)
 {
 	fprintf(stderr, "%s: %s\n", program, msg);
+	exit_tryhelp(status);
+}
+
+void
+exit_tryhelp(int status)
+{
+	fprintf(stderr, "Try `%s -h' for more information.\n", program);
 	exit(status);
 }
 
 void
-exit_usage()
+exit_printhelp()
 {
-	fprintf(stderr, "Usage: %s -A command options (accounting)\n", program);
-	fprintf(stderr, "       %s -B command options (blocking firewall)\n",
-		program);
-	fprintf(stderr, "       %s -F command options (forwarding firewall)\n",
-		program);
-	exit(2);
+	printf("Usage: %s -A command [options] (accounting)\n"
+		"       %s -B command [options] (blocking firewall)\n"
+		"       %s -F command [options] (forwarding firewall)\n"
+		"       %s -h (print this help information))\n\n",
+		program, program, program, program);
+
+	printf("Commands:\n"
+		"  -a [policy]	add rule (policy only required for firewall rules)\n"
+		"  -d [policy]	delete rule (policy only required for firewall rules)\n"
+		"  -l		list all rules of this category\n"
+		"  -z		reset packet/byte counters of all rules of this category\n"
+		"  -f		remove all rules of this category\n"
+		"  -p policy	change default policy (either accept, deny, or reject)\n"
+		"  -c		check acceptance of IP packet\n\n"
+		"Options:\n"
+		"  -P		protocol (either tcp, udp, icmp, or all)\n"
+		"  -S address[/mask] [port ...]\n"
+		"		source specification\n"
+		"  -D address[/mask] [port ...]\n"
+		"		destination specification\n"
+		"  -I address	interface address\n\n"
+		"  -b		bidirectional mode\n"
+		"  -e		extended output mode\n"
+		"  -k		turn on kernel logging for matching packets\n"
+		"  -n		numeric output of addresses and ports\n"
+		"  -v		verbose mode\n"
+		"  -x		expand numbers (display exact values)\n"
+		"  -y		match TCP packets with SYN set and ACK cleared\n");
+
+	exit(0);
 }
